library rw_tcx;

import 'dart:io';
import 'package:path_provider/path_provider.dart';

import 'models/TCXModel.dart';
import 'rTCX.dart';
import 'wTCX.dart';
import 'logTool.dart';

Future<File>  _localFile(String fileName) async {
  final directory = await getApplicationDocumentsDirectory();
  var path = directory.path;
  return File('$path/$fileName');
}

/// Generate the ride data structure
/// from a string
/// Not tested in detail because main goal
/// is to use writeTCX
///
TCXModel readTCX(String contents) {
  TCXModel rideData = TCXModel();
  rideData.points = List<TrackPoint>();

  // Search for the content of a tag
  Tag result = searchElement('Id', contents);
  displayInfo('result ${result.content}');


  Tag activity = searchAttribute('Activity', contents);
  displayInfo('Activity ${activity.content}');

  Tag totalDistance = searchElement('DistanceMeters', contents);
  displayInfo('Distance ${totalDistance.content}');

  Tag calories = searchElement('Calories', contents);
  displayInfo('Calories ${calories.content}');

  String speed = searchExtension('Speed', contents, index: 0);
  displayInfo('Speed $speed');

  // Start to store data in rideData structure
  rideData.totalDistance = double.tryParse(totalDistance.content);

  // search for sport= in the result and remove it
  List<String> _activity = activity.content.split('=');
  String activityType = _activity[1]; // Remove additional " "
  activityType = activityType.substring(1, activityType.length - 1);

  rideData.activityType = activityType;

  rideData.maxSpeed = double.tryParse(speed);
  rideData.calories = int.tryParse(calories.content);

  // Read now all the trackpoints
  //------------------------------
  bool noMoreTrackPoints = false;
  int idx = 0;

  

  do {
    TrackPoint trackPoint = extractTrackpoint(contents, idx);

    // Move to the next
    idx = trackPoint.index;
    // displayInfo(
        // 'idx : ${trackPoint.index}  -- ${trackPoint.timeStamp} -- ${trackPoint.distance}');
    if (trackPoint.index == null) {
      // No more trackpoint available in the String
      displayInfo('No more trackPoints!');
      noMoreTrackPoints = true;
    } else {
      // Add a new point in rideData

      // If position is missing skip this trackpoint
      // in garmin connect sometimes position is missing in trackpoint
      if ((trackPoint.latitude != null) &&
          (trackPoint.longitude != null) &&
          (trackPoint != null)) {
        rideData.points.add(trackPoint);
      } else {
        displayInfo('skip this position ${trackPoint.timeStamp}');
      }

      
    }
  } while (!noMoreTrackPoints);
  return rideData;
}







// Generate the TCX file
/// from a TCX Model
/// TODO: Add return code 
///
Future<void> writeTCX(TCXModel tcxInfos, String filename) async {
  // Now generate a new file from rideData
  var generatedTCXFile = await _localFile(filename);
  var sink = generatedTCXFile.openWrite(mode: FileMode.writeOnly);

  String contents = '';

  // Generate the prolog of the TCX file
  final String prolog = """ <?xml version="1.0" encoding="UTF-8"?>
    <TrainingCenterDatabase
    xsi:schemaLocation="http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2 http://www.garmin.com/xmlschemas/TrainingCenterDatabasev2.xsd"
    xmlns:ns5="http://www.garmin.com/xmlschemas/ActivityGoals/v1"
    xmlns:ns3="http://www.garmin.com/xmlschemas/ActivityExtension/v2"
    xmlns:ns2="http://www.garmin.com/xmlschemas/UserProfile/v2"
    xmlns="http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:ns4="http://www.garmin.com/xmlschemas/ProfileExtension/v1">\n""";



  final String tail  = """      <Creator xsi:type="Device_t">
        <Name>rw_TCX</Name>
        <UnitId>3873921795</UnitId>
        <ProductID>1736</ProductID>
        <Version>
          <VersionMajor>4</VersionMajor>
          <VersionMinor>20</VersionMinor>
          <BuildMajor>0</BuildMajor>
          <BuildMinor>0</BuildMinor>
        </Version>
      </Creator>
    </Activity>
  </Activities>
  <Author xsi:type="Application_t">
    <Name>Connect Api</Name>
    <Build>
      <Version>
        <VersionMajor>0</VersionMajor>
        <VersionMinor>0</VersionMinor>
        <BuildMajor>0</BuildMajor>
        <BuildMinor>0</BuildMinor>
      </Version>
    </Build>
    <LangID>en</LangID>
    <PartNumber>006-D2449-00</PartNumber>
  </Author>
  </TrainingCenterDatabase>""";


  String activityBiking = """<Activity Sport="Biking">\n""";

  String activitiesContent = '';

  // Add Activity
  //-------------
  String activityContent = activityBiking;

  // TODO to replace with real data
  // Not used
  var date = DateTime(2019, 6, 11, 18, 28);

  // Add ID
  activityContent =
      activityContent + addElement('Id', createTimestamp(date));

  displayInfo(' $activityContent');

  // Add lap
  //---------
  String lapContent = '';
  lapContent = lapContent + addElement('TotalTimeSeconds', '1688.0');
  // Add Total distace in meters
  lapContent = lapContent + addElement('DistanceMeters', '88888.0');
  // Add Maximum speed in meter/second
  lapContent = lapContent + addElement('MaximumSpeed', '12.8888');
  // Add calories
  lapContent = lapContent + addElement('Calories', '12.8888');
  // Add intensity (what is the meaning?)
  lapContent = lapContent + addElement('Intensity', 'Active');
  // Add intensity (what is the meaning?)
  lapContent = lapContent + addElement('TriggerMethod', 'Manual');

  // Add track inside the lap
  String trackContent = '';
  int counterTrackpoint = 0;

  for (var point in tcxInfos.points) {
    String trackPoint = addTrackPoint(point);
    counterTrackpoint++;

    // TODO: Remove the temp disp
    // Only the first 3 trackpoints
    //==============================
    if (counterTrackpoint < 4) {
      
      displayInfo(' $trackPoint');
    } // temp disp for the for loop

    trackContent = trackContent + trackPoint;
  }
  lapContent = lapContent + addElement('Track', trackContent);

  activityContent = activityContent +
      addAttribute('Lap', 'StartTime', createTimestamp(date), lapContent);

  activitiesContent = addElement('Activities', activityContent);


  // Create the complete tcx file
  contents = prolog + activitiesContent + tail;


  // sink = generatedTCXFile.openWrite(mode: FileMode.append);
  sink.write(contents);
  // Close the file
  await sink.flush();
  await sink.close();

// Add the tail of the TCX file

// Check what is the result of the file generation
  sink = generatedTCXFile.openWrite(mode: FileMode.append);

  contents = await generatedTCXFile.readAsString();
  // displayInfo(' $contents');
  displayInfo('The file is ${contents.length} bytes long.');
  
  // Close the file
  await sink.flush();
  await sink.close();
}
